@@ -101,6 +101,64 @@ client_deliver_wm_protocol(Client *c, Atom delivery)
 			 delivery, CurrentTime, 0, 0, 0);
 }
 
+void
+client_get_wm_protocols(Client *c)
+{
+  Atom  *protocols = NULL;
+  int    n = 0, i = 0;
+  Status status;
+  
+   /* Check for 'special' extra button/ping protocols */
+
+  misc_trap_xerrors();
+
+  status = XGetWMProtocols(c->wm->dpy, c->window, &protocols, &n);
+
+  if (status && n && !misc_untrap_xerrors()) 
+    {
+      dbg("%s() checking wm protocols ( %i found )\n", __func__, n);
+
+      c->flags &= ~(CLIENT_HELP_BUTTON_FLAG|CLIENT_ACCEPT_BUTTON_FLAG|CLIENT_CUSTOM_BUTTON_FLAG);
+
+      for (i=0; i<n; i++)
+	{
+	  if (protocols[i] == c->wm->atoms[_NET_WM_CONTEXT_HELP])
+	    {
+	      dbg("%s() got _NET_WM_CONTEXT_HELP protocol\n", __func__ );
+	      c->flags |= CLIENT_HELP_BUTTON_FLAG;
+	    }
+	  else if (protocols[i] == c->wm->atoms[_NET_WM_CONTEXT_ACCEPT])
+	    {
+	      dbg("%s() got _NET_WM_CONTEXT_ACCEPT protocol\n", __func__ );
+	      c->flags |= CLIENT_ACCEPT_BUTTON_FLAG;
+	    }
+	  else if (protocols[i] == c->wm->atoms[_NET_WM_CONTEXT_CUSTOM])
+	    {
+	      dbg("%s() got _NET_WM_CONTEXT_CUSTOM protocol\n", __func__ );
+	      c->flags |= CLIENT_CUSTOM_BUTTON_FLAG;
+	    }
+#ifndef NO_PNG
+	  else if (protocols[i] == c->wm->atoms[_NET_WM_PING]
+		   && c->host_machine && c->pid)
+	    {
+	      dbg("%s() has PING ewmh\n", __func__);
+	      c->has_ping_protocol = True;
+	    }
+#endif
+#ifdef USE_XSYNC
+	  else if (protocols[i] == c->wm->atoms[_NET_WM_SYNC_REQUEST])
+	    {
+	      c->has_ewmh_sync = True;
+	      dbg("%s() client has _NET_WM_SYNC_REQUEST\n", __func__);
+	    }
+#endif
+	}
+    }
+
+  if (protocols)
+    XFree(protocols);
+}
+
 void
 client_deliver_message(Client       *c, 
 		       Atom          delivery_atom,
@@ -334,20 +392,72 @@ client_get_transient_list(Wm *w, MBList **list, Client *c)
     }
 }
 
-Client*
-client_get_highest_transient(Client *c, int client_flags)
+static Client*
+client_get_highest_transient_recurse (Client *c, 
+				      int     client_flags, 
+				      Client *ignore,
+				      int    *depth)
 {
   Wm     *w = c->wm;
   Client *p = NULL;
-  Client *highest = c;
+  Client *highest = c, *tmp;
+  int     this_depth = 0, max_depth = 0;
 
+  /* FIXME: its likely this can be combined into
+   * client_get_highest_transient() somehow.. 
+  */
   stack_enumerate(w,p)
     {
-      if (p != c && p->trans && p->trans == c)
+      if (p != c && p->trans && p->trans == c && p != ignore)
 	{
 	  if (client_flags && !(p->flags & client_flags))
 	    continue;
-	  highest = client_get_highest_transient(p, client_flags);
+
+	  this_depth++;
+
+	  tmp = client_get_highest_transient_recurse(p, 
+						     client_flags, 
+						     ignore, 
+						     &this_depth);
+	  if (this_depth > max_depth)
+	    {
+	      max_depth = this_depth;
+	      highest = tmp;
+	    }
+	}
+    }
+
+  *depth += max_depth;
+
+  return highest;
+}
+
+Client*
+client_get_highest_transient(Client *c, int client_flags, Client *ignore)
+{
+  Wm     *w = c->wm;
+  Client *p = NULL;
+  Client *highest = c, *tmp;
+  int     depth = 0, depth_max = 0;
+
+  stack_enumerate(w,p)
+    {
+      if (p != c && p->trans && p->trans == c && p != ignore)
+	{
+	  depth = 0;
+
+	  /* Recurse for each directly transient client, getting the depth 
+	   * for any extra matchbox children.
+	  */
+	  tmp = client_get_highest_transient_recurse (p, 
+						      client_flags,
+						      ignore,
+						      &depth);
+	  if (depth >= depth_max)
+	    {
+	      highest = tmp;
+	      depth_max = depth;
+	    }
 	}
     }
 
