@@ -218,10 +218,41 @@ dialog_client_show(Client *c)
 
   if (!c->mapped)
     {
-      XMapSubwindows(w->dpy, c->frame);
-      XMapWindow(w->dpy, c->frame);
+      if (c->flags & CLIENT_IS_MINIMIZED)
+	{
+	  Client *p = NULL;
+
+	  client_set_state(c, NormalState);
+	  c->flags &= ~CLIENT_IS_MINIMIZED;
+
+	   /* Make sure any transients are un minimized too */
+	   stack_enumerate(w, p)
+	     if (p->trans == c)
+	       {
+		 p->show(p);
+		 XMapSubwindows(w->dpy, p->frame);
+		 XMapWindow(w->dpy, p->frame);
+	       }
+	}
+
+      if (c->win_modal_blocker)
+	{
+	  XMapWindow(w->dpy, c->win_modal_blocker);
+	  dbg("%s() blocker win mapped for '%s'\n", __func__, c->name);
+	}
+
+      /* Set flag below to delay actual mapping of window 
+       * till *after* we've synced the stack. This hacky  
+       * solution avoids problem of dialogs mapping for
+       * clients below and causing flicker.
+       *
+       * Note could still cause issues with transients and remmapping
+       * them if minimised.. 
+      */
+      c->flags |= CLIENT_DELAY_MAPPING;
     }
 
+
   /* 
    *  We just need to get the order right in respect 
    *  to other dialogs transient to the same app or 
@@ -234,6 +265,7 @@ dialog_client_show(Client *c)
     {
       /* Were transient for something 
        * - so recursives find the highest transient for this app
+       *   ignoring current client.
        * - raise ourselves above 
        */
       Client *lowest_trans = c->trans;
@@ -243,7 +275,9 @@ dialog_client_show(Client *c)
       while (lowest_trans->trans != NULL) 
 	lowest_trans = lowest_trans->trans;
 
-      highest_client = client_get_highest_transient(lowest_trans, urgent_flag);
+      highest_client = client_get_highest_transient(lowest_trans, 
+						    urgent_flag,
+						    c);
 
       if (c->mapped && highest_client == c)
 	{
@@ -286,7 +320,6 @@ dialog_client_show(Client *c)
 
   list_destroy(&transient_list);
 
-
   /* Insurance below */
 
   if (wm_get_visible_main_client(w))
@@ -300,7 +333,6 @@ dialog_client_show(Client *c)
   stack_move_transients_to_top(w, NULL, CLIENT_HAS_URGENCY_FLAG);
 
   c->mapped = True;
-
 }
 
 void
@@ -376,6 +408,7 @@ dialog_client_reparent(Client *c)
      }
 
    if (c->flags & CLIENT_IS_MODAL_FLAG
+       && w->config->super_modal
        && c->trans == NULL )	/* modal for device. XXX check recursive ? */
      {
        /* Create an InputOnly fullscreen window to aid in making 
@@ -394,12 +427,12 @@ dialog_client_reparent(Client *c)
 					    CWOverrideRedirect|CWEventMask,
 					    &attr);
 
-       XMapWindow(w->dpy, c->win_modal_blocker);
+       /* Map is show() or gets stacked above dialog */
+       /* XMapWindow(w->dpy, c->win_modal_blocker); */
 
-       dbg("%s() created blocked win\n", __func__);
+       dbg("%s() created and mapped blocker win for '%s'\n", __func__, c->name);
        
-       w->stack_n_items++;
-
+       w->n_modal_blocker_wins++;
      }
 
    XClearWindow(w->dpy, c->frame);
@@ -1194,6 +1227,12 @@ dialog_client_iconize(Client *c)
   c->mapped = False;
   XUnmapWindow(w->dpy, c->frame); 
 
+  if (c->win_modal_blocker)
+    {
+      XUnmapWindow(w->dpy, c->win_modal_blocker);
+      dbg("%s() blocker win unmapped for '%s'\n", __func__, c->name);
+    }
+
   /* Make sure any transients get iconized too */  
   stack_enumerate(w, p)
     if (p->trans == c)
@@ -1215,7 +1254,8 @@ dialog_client_destroy(Client *c)
     {
       Wm *w = c->wm;
       XDestroyWindow(w->dpy, c->win_modal_blocker);
-      w->stack_n_items--;
+       w->n_modal_blocker_wins--;
+       dbg("%s() blocker win destroyed for '%s'\n", __func__, c->name);
     }
 
   base_client_destroy(c);
