@@ -265,21 +265,13 @@ ewmh_handle_root_message(Wm *w, XClientMessageEvent *e)
    return 0;
 }
 
-void
-ewmh_update(Wm *w)
-{
-  ewmh_update_lists(w);
-  ewmh_update_rects(w);
-}
-
 void
 ewmh_update_lists(Wm *w)
 {
    Client        *c = NULL;
    Window        *wins = NULL;
    int            cnt = 0;
-   unsigned long  val[1];
-
+   
    dbg("%s(): called %i\n", __func__, n_stack_items(w)); 
 
 #ifdef USE_LIBSN
@@ -384,15 +376,20 @@ ewmh_update_lists(Wm *w)
 		     (unsigned char *)wins, n_stack_items(w));
    }
 
+  if (wins)
+    free(wins);
+}
+
+void
+ewmh_update_desktop_hint(Wm *w)
+{
    /* Desktop showing hint */
 
-   val[0] = (w->flags & DESKTOP_RAISED_FLAG) ? 1 : 0;
+   int val = (w->flags & DESKTOP_RAISED_FLAG) ? 1 : 0;
 
    XChangeProperty(w->dpy, w->root, w->atoms[_NET_SHOW_DESKTOP],
 		   XA_CARDINAL, 32, PropModeReplace, 
-		   (unsigned char *)val, 1);
-  if (wins)
-    free(wins);
+		   (unsigned char *)&val, 1);
 }
 
 void
@@ -453,6 +450,11 @@ ewmh_state_set(Client *c)
     XChangeProperty(w->dpy, c->window, w->atoms[WINDOW_STATE],
 		    XA_ATOM, 32, PropModeReplace, 
 		    (unsigned char *)atom_states, n_atom_states);
+  else
+    XChangeProperty(w->dpy, c->window, w->atoms[WINDOW_STATE],
+		    XA_ATOM, 32, PropModeReplace, 
+		    (unsigned char *)NULL, 0);
+
 }
 
 Bool 
@@ -553,7 +555,7 @@ ewmh_set_current_app_window(Wm *w)
 }
 
 
-#define PING_PENDING_MAX 2
+
 
 void
 ewmh_hung_app_check(Wm *w)
@@ -589,9 +591,38 @@ ewmh_hung_app_check(Wm *w)
 	  e.xclient.data.l[1] = c->window;
 	  XSendEvent(w->dpy, c->window, False, 0, &e);
 
-	  if (c->pings_pending >= PING_PENDING_MAX)
+	  if (c->pings_pending > PING_PENDING_MAX)
 	    {
-	      client_obliterate(c);
+	      if (w->config->ping_handler && c->pid)
+		{
+		  /* fire off external binary to handle hung app 
+                   * if env var is set. 
+		  */
+
+		  int   len;
+		  char *buf = NULL;
+
+		  len = strlen(w->config->ping_handler) + 32;
+		  buf = malloc(len);
+
+		  if (buf)
+		    {
+		      snprintf(buf, len-1, "%s %i %li",
+			       w->config->ping_handler,
+			       c->pid,
+			       c->window);
+		  
+		      fork_exec(buf);
+
+		      free(buf);
+		    }
+
+		  /* dont ping any more */
+		  c->pings_pending = -1;
+		  w->n_active_ping_clients--;
+		}
+	      else
+		client_obliterate(c);
 	    }
 	}
     }
@@ -666,7 +697,7 @@ ewmh_get_utf8_prop(Wm *w, Window win, Atom req_atom)
 {
   Atom           type;
   int            format, result;
-  long           bytes_after, n_items;
+  unsigned long  bytes_after, n_items;
   unsigned char *str = NULL;
 
   misc_trap_xerrors();
@@ -709,7 +740,7 @@ ewmh_get_icon_prop_data(Wm *w, Window win)
 {
   Atom           type;
   int            format, result;
-  long           bytes_after, n_items;
+  unsigned long  bytes_after, n_items;
   unsigned char *data = NULL;
 
   misc_trap_xerrors();
