@@ -82,6 +82,8 @@ main_client_manage_toolbars_for_fullscreen(Client *c, Bool main_client_showing)
   Wm     *w = c->wm;
   Client *p  = NULL;
 
+  dbg("%s() called\n", __func__);
+
   stack_enumerate(w, p)
     {
       if (p->type == MBCLIENT_TYPE_DIALOG)
@@ -96,6 +98,8 @@ main_client_manage_toolbars_for_fullscreen(Client *c, Bool main_client_showing)
 	    }
 	  else if (p->flags & CLIENT_TB_ALT_TRANS_FOR_DIALOG)
 	    {
+	      dbg("%s() IM trans for dialog found resizing..\n", __func__);
+
 	      toolbar_client_configure(p);
 	      toolbar_client_move_resize(p);
 	      return 0;
@@ -300,7 +304,7 @@ main_client_reparent(Client *c)
   XClearWindow(w->dpy, c->frame);
 
   XSetWindowBorderWidth(w->dpy, c->window, 0);
-  XAddToSaveSet(w->dpy, c->window);
+  XAddToSaveSet(w->dpy, c->window); 
   XSelectInput(w->dpy, c->window, ColormapChangeMask|PropertyChangeMask);
   XReparentWindow(w->dpy, c->window, c->frame, offset_west, offset_north);
 }
@@ -584,22 +588,33 @@ void main_client_button_press(Client *c, XButtonEvent *e)
       return;
    }
 
-   stack_enumerate(w, p)
+   if (w->config->super_modal)
      {
-       if (p->trans == c && (p->flags & CLIENT_IS_MODAL_FLAG))
+       stack_enumerate(w, p)
 	 {
-	   MBList *button_item = client_get_button_list_item_from_event(c, e);
-
-	   /* In the precense of a modal transient dialog ignore 
-	    * certain buttons. 
-	    *
-	    * For now its just the custom type. May make sense to
-            * Add more.
-	   */
-	   if ((button_item && button_item->id == BUTTON_ACTION_CUSTOM)
-	       || (button_item && button_item->id == BUTTON_ACTION_MIN)
-	       || (button_item && button_item->id == BUTTON_ACTION_CLOSE))
-	     return;
+	   if (p->trans == c && (p->flags & CLIENT_IS_MODAL_FLAG))
+	     {
+	       MBList *button_item = client_get_button_list_item_from_event(c, e);
+	       /* In the precense of a modal transient dialog ignore 
+		* certain buttons. 
+		*
+		*/
+	       if ((button_item && button_item->id == BUTTON_ACTION_CUSTOM)
+		   || (button_item && button_item->id == BUTTON_ACTION_MIN)
+		   || (button_item && button_item->id == BUTTON_ACTION_CLOSE))
+		 {
+		   if (button_item->id == BUTTON_ACTION_CLOSE)
+		     {
+		       /* initiate pinging the app anyway for close button */
+		       if (c->has_ping_protocol && c->pings_pending == -1) 
+			 {
+			   c->pings_pending = 0;
+			   w->n_active_ping_clients++;
+			 }
+		     }
+		   return;
+		 }
+	     }
 	 }
      }
 
@@ -737,6 +752,8 @@ main_client_iconize(Client *c)
   Wm *w = c->wm;
   Client *p = NULL;
 
+  dbg("%s() called on %s\n", __func__, c->name);
+
   client_set_state(c, IconicState);
   c->flags |= CLIENT_IS_MINIMIZED;
 
@@ -782,9 +799,18 @@ main_client_show(Client *c)
      {
        if (c->flags & CLIENT_IS_MINIMIZED)
 	 {
+	   Client *p = NULL;
+
+	   /* Reset state from Iconized */
 	   client_set_state(c, NormalState);
 	   c->flags &= ~CLIENT_IS_MINIMIZED;
+
+	   /* Make sure any dialogs are shown too */
+	   stack_enumerate(w, p)
+	     if (p->trans == c)
+	       p->show(p);
 	 }
+
        XMapSubwindows(w->dpy, c->frame);
        XMapWindow(w->dpy, c->frame);
      }
