@@ -111,7 +111,7 @@ lookup_frame_type(char *name)
       return frame_lookup[i].id;
   return -1;
 }
-
+ 
 void theme_frame_icon_paint(MBTheme *t, Client *c, 
 			    MBPixbufImage *img_dest, 
 			    int x, int y)
@@ -131,15 +131,16 @@ void theme_frame_icon_paint(MBTheme *t, Client *c,
     {
       if (c->icon)
 	{
-	  Window win_crap;
-	  int icon_w, icon_h, crap;
-	  XGetGeometry(t->wm->dpy, c->icon, &win_crap, &crap, &crap, 
-		       &icon_w, &icon_h, &crap, &crap);
+	  Window       win_foo;
+	  int          foo;
+	  unsigned int icon_w, icon_h, ufoo;
+	  XGetGeometry(t->wm->dpy, c->icon, &win_foo, &foo, &foo, 
+		       &icon_w, &icon_h, &ufoo, &ufoo);
 	  img = mb_pixbuf_img_new_from_drawable(t->wm->pb, 
 						c->icon, 
 						c->icon_mask, 
 						0, 0, 
-						icon_w, icon_h);
+						(int)icon_w, (int)icon_h);
 	} 
     }
 
@@ -864,7 +865,8 @@ theme_frame_menu_get_dimentions(MBTheme* theme, int *w, int *h)
   stack_enumerate(theme->wm, p)
   {
     if ((p->type == MBCLIENT_TYPE_APP || p->type == MBCLIENT_TYPE_DESKTOP) 
-	&& p->name && p->mapped
+	&& p->name 
+	/* && p->mapped */  /* Include iconsized clients in sizing */
 	&& p != wm_get_visible_main_client(theme->wm))
       {
 	int this_width = mb_font_get_txt_width (frame->font,
@@ -1091,7 +1093,7 @@ theme_frame_menu_paint(MBTheme* theme, Client *c)
       p = (Client*)item->data;
       if (p->type == MBCLIENT_TYPE_APP 
 	  && p->name && !(p->flags & CLIENT_IS_DESKTOP_FLAG)
-	  && p->mapped
+	  && p->mapped 
 	  && p != wm_get_visible_main_client(w))
 	{
 	  theme_frame_icon_paint(theme, p, img, 
@@ -2272,27 +2274,10 @@ void
 mbtheme_switch (Wm   *w, 
 		char *new_theme_name)
 {
-  Client *c, *p, *orig;
-  Window win_active = None;
-  Bool reset_new_for_desktop_flag = False;
+  Client *p = NULL;
 
   XGrabServer(w->dpy);
 
-  orig = c = p = w->stack_top_app; 
-
-  if (w->stack_top_app)
-    {
-      win_active = w->stack_top_app->window;
-      if (w->stack_top_app->flags & CLIENT_NEW_FOR_DESKTOP)
-	reset_new_for_desktop_flag = True;
-    }
-
-  if (w->flags & DESKTOP_RAISED_FLAG
-      && (orig = wm_get_desktop(w)))
-    {
-      win_active = orig->window;
-    }
-
   /* now the fun part */
   mbtheme_free(w, w->mbtheme);
 
@@ -2316,6 +2301,8 @@ mbtheme_switch (Wm   *w,
 	}
     }
 
+  /* Now resize ( due to new frames ) + repaint everything */
+
   stack_enumerate(w, p)
     {
       client_buttons_delete_all(p);
@@ -2328,8 +2315,6 @@ mbtheme_switch (Wm   *w,
 	  /*  Old theme may have been shaped and this one not. 
 	   *  Therefore we 'clear' any possible shapes set on 
 	   *  the window frames.  
-	   *  Im not sure if this is the best way to do this but
-	   *  SHAPE docs serverely lacking :(
 	   */
 	  
 	  xregion = XCreateRegion ();
@@ -2355,32 +2340,13 @@ mbtheme_switch (Wm   *w,
       p->move_resize(p);
       p->redraw(p, False);
     }
-    
 
-  /* fix if desktop is shown */
-  if (win_active)
-    {
-      Client *found = wm_find_client(w, win_active, WINDOW);
-      if (found)
-	{
-	  if (found->type == MBCLIENT_TYPE_DESKTOP)
-	    {
-	      w->flags &= ~DESKTOP_RAISED_FLAG; /* Clear desktop flag */
-	      wm_toggle_desktop(w);
-	    } 
-	  else 
-	    {
-	      wm_activate_client(found);
-	      if (reset_new_for_desktop_flag)
-		found->flags |= CLIENT_NEW_FOR_DESKTOP;
-	    }
-	}
-    }
+  ewmh_update_rects(w); /* theme *could* affect this */
+    
+  XSync(w->dpy, False);
 
   XUngrabServer(w->dpy);
 
-  XSync(w->dpy, False);
-
   comp_engine_render(w, None);
 }
 
@@ -2489,10 +2455,10 @@ mbtheme_init (Wm   *w,
   chdir(theme_path);
 
   XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME], XA_STRING, 8,
-		  PropModeReplace, theme_path, strlen(theme_path));
+		  PropModeReplace, (unsigned char*)theme_path, strlen(theme_path));
 
   XChangeProperty(w->dpy, w->root, w->atoms[_MB_THEME_NAME], XA_STRING, 8,
-		  PropModeReplace, theme_name, strlen(theme_name));
+		  PropModeReplace, (unsigned char*)theme_name, strlen(theme_name));
 
   free(theme_path);
 
