@@ -293,6 +293,7 @@ typedef struct _mb_client_button
 #define CLIENT_HAS_URGENCY_FLAG (1<<26)
 #define CLIENT_HAS_ABOVE_STATE (1<<27)
 #define CLIENT_IS_MENU_DIALOG ( 1<<30)
+#define CLIENT_DELAY_MAPPING  ( 1<<31)
 
 #ifdef USE_ALT_INPUT_WIN
 #define CLIENT_TB_ALT_TRANS_FOR_DIALOG (1<<28)
@@ -477,6 +478,7 @@ typedef struct _wm_config
   Bool         no_cursor;
   int          dialog_stratergy;
 
+  Bool         super_modal;
   Bool         dialog_shade; 
   int          lowlight_params[4];
 
@@ -562,6 +564,8 @@ typedef struct _wm
 
   MBList           *client_age_list; /* List of clients ordered by age */
 
+  int               n_modal_blocker_wins; /* needed for restack() call */
+
   /*******************/
 
   Wm_config        *config;  
