@@ -186,14 +186,12 @@ wm_usage(char *progname)
 #endif
    printf("\t-use_dialog_mode  <free|static|const-horiz>\n");
    printf("\t-use_desktop_mode <decorated|plain>\n");
+   printf("\t-use_super_modal  <yes|no>\n");
    printf("\t-force_dialogs    <comma seperated list of window titles>\n");
 #ifdef USE_SM
    printf("\t--sm-client-id    <session id>\n");
 #endif
 
-   /*
-   printf("\t-ping_handler     <string>\n");
-   */
 #ifdef STANDALONE
    printf("\t-titlebar_panel   <x11 geometry>\n");
 #endif
@@ -311,7 +309,7 @@ wm_load_config (Wm   *w,
    w->config->no_cursor        = False;
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
-   w->config->ping_handler     = NULL;
+   w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
 
 #ifdef USE_COMPOSITE
    w->config->dialog_shade = True;
@@ -365,6 +363,13 @@ wm_load_config (Wm   *w,
 	  continue;
 	}
 #endif
+      if (!strcmp ("-use_super_modal", argv[i])) 
+	{
+	  if (++i>=*argc) wm_usage (argv[0]);
+	  if (strcmp(argv[i], "yes") == 0)
+            w->config->super_modal = True;   
+	  continue;
+	}
       if (!strcmp("-use_dialog_mode", argv[i])) 
 	{
 	  if (++i>=*argc) wm_usage (argv[0]);
@@ -449,7 +454,7 @@ wm_load_config (Wm   *w,
    char              *type;
    XrmValue          value;
    
-   static int opTableEntries = 10;
+   static int opTableEntries = 11;
    static XrmOptionDescRec opTable[] = {
       {"-theme",       ".theme",           XrmoptionSepArg, (XPointer) NULL},
       {"-use_titlebar",".titlebar",        XrmoptionSepArg, (XPointer) NULL},
@@ -461,6 +466,7 @@ wm_load_config (Wm   *w,
       {"-titlebar_panel",  ".titlebarpanel", XrmoptionSepArg, (XPointer) NULL},
       {"-force_dialogs",  ".forcedialogs", XrmoptionSepArg, (XPointer) NULL},
       {"--sm-client-id",  ".session",      XrmoptionSepArg, (XPointer) NULL},
+      {"-use_super_modal", ".supermodal",  XrmoptionSepArg, (XPointer) NULL},
    };
 
    XrmInitialize();
@@ -482,7 +488,7 @@ wm_load_config (Wm   *w,
    w->config->no_cursor        = False;
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
-   w->config->ping_handler     = NULL;
+   w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
 
    if (XrmGetResource(rDB, "matchbox.display",
 		      "Matchbox.Display",
@@ -554,6 +560,17 @@ wm_load_config (Wm   *w,
 	 }
      }   
 
+   if (XrmGetResource(rDB, "matchbox.supermodal", "Matchbox.Supermodal",
+		      &type, &value) == True)
+   {
+      if(strncmp(value.addr, "yes", (int) value.size) == 0)
+      {
+	 dbg("%s() TURNING SUPER MODAL ON\n", __func__);
+	 w->config->super_modal = True;
+      }
+   }
+
+
    /* 
     *  Composite matchbox always uses lowlighting 
     */
@@ -654,6 +671,11 @@ wm_init_existing(Wm *w)
    XWindowAttributes attr;
    Client *c;
    
+   /* set blank hints */
+   ewmh_update_rects(w); 
+   ewmh_update_lists(w); 
+   ewmh_update_desktop_hint(w);
+
    XQueryTree(w->dpy, w->root, &dummyw1, &dummyw2, &wins, &nwins);
    for (i = 0; i < nwins; i++) {
       XGetWindowAttributes(w->dpy, wins[i], &attr);
@@ -922,8 +944,8 @@ wm_event_loop(Wm* w)
 
 
 #ifndef NO_PING
-	/* check for hung apps every two seconds - they dont last long.. */
-	if (++hung_app_timer > 2 && w->n_active_ping_clients)
+	/* check for hung apps every PING_CHECK_FREQ seconds */
+	if (++hung_app_timer > PING_CHECK_FREQ && w->n_active_ping_clients)
 	  {
 	    hung_app_timer = 0;
 	    ewmh_hung_app_check(w);
@@ -1281,6 +1303,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
      {
        /* Dont know any thing about this window. Let it have what it 
         * wants and leave. 
+	* Can be a window initally mapping, thus the trap as it could
+        * of already dissapeared.
         */
        dbg("%s() could find source client %ix%i\n", __func__,
 	   e->width, e->height);
@@ -1290,7 +1314,9 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
        xwc.height  = req_h;
        xwc.sibling = e->above;
        xwc.stack_mode = e->detail;
+       misc_trap_xerrors();
        XConfigureWindow(w->dpy, e->window, e->value_mask, &xwc);
+       misc_untrap_xerrors();
        return;
      }
 
@@ -1689,7 +1715,25 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
       if (!misc_untrap_xerrors() && success)
 	if ((new_trans_client = wm_find_client(w, trans_win, WINDOW)) != NULL)
 	  {
+	    Client *p = new_trans_client;
+
+	    /* Dont let clients be change transiency to themselves
+             * either directly or recursively.
+	    */
+	    while (p != NULL)
+	      {
+		if (p == c)
+		  {
+		    dbg("%s() CLIENT WARNING: %s ( %li ) transient for self\n",
+			__func__, c->name, c->window);
+		    c->trans = NULL;
+		    return; 
+		  }
+		p = p->trans;
+	      }
+
 	    c->trans = new_trans_client;
+
 	    return;
 	  }
 
@@ -1706,7 +1750,7 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 
       misc_trap_xerrors(); 	/* avoid possible X Errors */
 
-      c->name = ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
+      c->name = (char*)ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
 
       if (!misc_untrap_xerrors())
 	{
@@ -1723,11 +1767,35 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
 	  update_titlebar = True;
 	}
     }
+  else  if (e->atom == w->atoms[WM_PROTOCOLS])
+    {
+      int orig_flags = c->flags;
+
+      dbg("%s() WM_PROTOCOLS changed, flags %i\n", __func__, orig_flags);
+
+      client_get_wm_protocols(c);
+
+      dbg("%s() WM_PROTOCOLS changed, flags now %i\n", __func__, c->flags);
+
+      /* If flags have changed, likely means a WM_CONTEXT_HELP|etc has changed 
+       * and thus titlebar needs a repaint.
+      */
+      if (c->flags != orig_flags)
+	{
+	  dbg("%s() WM_PROTOCOLS changed flags, repainting\n", __func__);
+
+	  /* Buttons will get recreated on demand via repaint */
+	  client_buttons_delete_all(c);
+	  update_titlebar = True;
+	}
+    }
   else  if (e->atom == w->atoms[WM_CHANGE_STATE])
     {
       dbg("%s() state change, name is %s\n", __func__, c->name);
       if(client_get_state(c) == WithdrawnState)
-	c->destroy(c);
+	{
+	  wm_remove_client(w, c);
+	}
     }
   else if (e->atom == w->atoms[CM_TRANSLUCENCY])
     {
@@ -1765,7 +1833,7 @@ wm_make_new_client(Wm *w, Window win)
    unsigned long n, extra;
    int           format, status;
    Client       *c = NULL, *t = NULL;
-   XWMHints     *wmhints;
+   XWMHints     *wmhints = NULL;
    int           mwm_flags = 0;
 
    XGrabServer(w->dpy);
@@ -1785,10 +1853,16 @@ wm_make_new_client(Wm *w, Window win)
      {
        /* Use EWMH Window Type Hint to figure out window type */
 
+       misc_trap_xerrors();
+
        status = XGetWindowProperty(w->dpy, win, w->atoms[WINDOW_TYPE], 
 				   0L, 1000000L, 0, XA_ATOM, 
 				   &realType, &format,
 				   &n, &extra, (unsigned char **) &value);
+
+       if (misc_untrap_xerrors()) /* An X error occured - win deleted ? */
+	 goto end;
+       
        if (status == Success)
 	 {
 	   if (realType == XA_ATOM && format == 32)
@@ -1889,6 +1963,7 @@ wm_make_new_client(Wm *w, Window win)
 		     break; 
 		   }
 	    }
+	    if (wmhints) XFree(wmhints);
 	 }
       }
 
@@ -1989,7 +2064,17 @@ wm_remove_client(Wm *w, Client *c)
    * If below is not done - suttle bugs appear with toolkits ( GTK )
    */
   client_set_state(c, WithdrawnState);
-  XReparentWindow(w->dpy, c->window, w->root, 0, 0); 
+
+  /* Below is likely not needed ( window will have already unmapped )
+   * and likely to fire an X error, but we do it anyway to be extra safe.
+   * 
+   * In gtk quickly hiding/showing window repeatadly it can seem gtk does't
+   * always call the unmap and a wierd ghost window is left.
+  */
+  XUnmapWindow(w->dpy, c->window);
+
+  /* Reparent now unmanaged win to root */
+  XReparentWindow(w->dpy, c->window, w->root, c->x, c->y); 
   XRemoveFromSaveSet(w->dpy, c->window);
 
   /* sync here so any (likely) lingering X errors are trapped */
@@ -2243,10 +2328,11 @@ wm_update_layout(Wm         *w,
        }
    }
 
+ ewmh_update_rects(w);
+
  XSync(w->dpy, False);
  XUngrabServer(w->dpy);
 
- ewmh_update_rects(w);
 }
 
 
@@ -2289,6 +2375,7 @@ wm_activate_client(Client *c)
 {
   Wm     *w;
   Client *client_to_focus = c;
+  Bool    set_desktop_show_hint = False;
 
   if (c == NULL) return; /* its possible for this to happen :( */
 
@@ -2311,10 +2398,21 @@ wm_activate_client(Client *c)
 	 We need to sync extra stuff up when displaying a new one.
        */
 
+      Bool switching_from_to_fullscreen = False;
+
       /* save focus state for transient dialogs of prev showing main win */
 
       if (w->stack_top_app && w->stack_top_app != c)
-	w->stack_top_app->hide(w->stack_top_app); 
+	{
+	  if ( (w->stack_top_app->flags & CLIENT_FULLSCREEN_FLAG
+		&& !(c->flags & CLIENT_FULLSCREEN_FLAG))
+	       || 
+	       ( !(w->stack_top_app->flags & CLIENT_FULLSCREEN_FLAG)
+		&& c->flags & CLIENT_FULLSCREEN_FLAG ) )
+	    switching_from_to_fullscreen = True;
+
+	  w->stack_top_app->hide(w->stack_top_app); 
+	}
 
       /* If this client has a saved dialog focus state, load it */
 
@@ -2354,6 +2452,9 @@ wm_activate_client(Client *c)
 	  dbg("%s() clearing desktop flag\n", __func__);
 	  w->flags &= ~DESKTOP_RAISED_FLAG;
 	  w->stack_top_app = c;      
+
+	  set_desktop_show_hint = True;
+
 	}
       else
 	{
@@ -2368,11 +2469,23 @@ wm_activate_client(Client *c)
 	    {
 	      stack_move_above_client(w->have_titlebar_panel, NULL);
 	    }
+
+	  set_desktop_show_hint = True;
+
 	}
 
       /* Set active main client, set focus handles active win hint */
       ewmh_set_current_app_window(w);
 
+#ifdef USE_ALT_INPUT_WIN
+      /* If switching from/to fullscreen, then there could be an 
+       * input window transient for a transient for root dialog and
+       * it therefore needs to be resized ( the IM ).
+      */
+      if (switching_from_to_fullscreen &&  c->type == MBCLIENT_TYPE_APP)
+	  main_client_manage_toolbars_for_fullscreen(c, False);
+#endif
+
     }
   else if (c->type == MBCLIENT_TYPE_DIALOG)
     {
@@ -2408,8 +2521,29 @@ wm_activate_client(Client *c)
 	  */
 	  wm_stack_dialogs_for_transient(w, wm_get_visible_main_client(w));
 	}
-    }
 
+      /* Check if this is transient for an app/desktop thats
+	 not currently at the top of the stack. In which case
+	 dont try and focus it.
+      */
+      if (c->trans != NULL)     /* not trans for root */
+	{
+	  Client *dialog_parent = c->trans;
+	  
+	  while (dialog_parent->trans != NULL)
+	    dialog_parent = dialog_parent->trans;
+	 
+	  
+	  if ( (dialog_parent->type == MBCLIENT_TYPE_APP
+		|| dialog_parent->type == MBCLIENT_TYPE_DESKTOP)
+	       && wm_get_visible_main_client(w) != dialog_parent)
+	  {
+		  client_to_focus = NULL;
+	  }
+      	}
+    
+  
+    }
 
   /* Always make sure embedded titlebar panels arn't visible for desktop 
    */
@@ -2426,9 +2560,10 @@ wm_activate_client(Client *c)
       stack_move_above_client(c, NULL);
     }
 
-  ewmh_update(w);
+  ewmh_update_lists(w);
 
-  client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
+  if (set_desktop_show_hint) 
+    ewmh_update_desktop_hint(w);
 
   stack_dump(w);
 
@@ -2436,6 +2571,15 @@ wm_activate_client(Client *c)
 
   stack_sync_to_display(w);
 
+  if (c->flags & CLIENT_DELAY_MAPPING) /* See dialog show() source */
+    {
+      c->flags &= ~CLIENT_DELAY_MAPPING;
+      XMapSubwindows(w->dpy, c->frame);
+      XMapWindow(w->dpy, c->frame);
+    }
+
+  client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
+
   comp_engine_client_show(w, c);
 
   XSync(w->dpy, False);	    
@@ -2544,6 +2688,8 @@ wm_toggle_desktop(Wm *w)
        dbg("%s() showing desktop\n", __func__);
        wm_activate_client(wm_get_desktop(w));
      }
+
+
 }
 
 void
